@@ -5,7 +5,7 @@
 #![forbid(unsafe_code)]
 
 // #![allow(unused_imports)]
-const SQLITE_64BIT_ERROR: &str = r##"SQLite cannot natively store unsigned 64-bit integers, so Turbosql does not support u64 fields. Use i64, u32, f64, or a string or binary format instead. (see https://github.com/trevyn/turbosql/issues/3 )"##;
+const SQLITE_U64_ERROR: &str = r##"SQLite cannot natively store unsigned 64-bit integers, so Turbosql does not support u64 fields. Use i64, u32, f64, or a string or binary format instead. (see https://github.com/trevyn/turbosql/issues/3 )"##;
 
 use once_cell::sync::Lazy;
 use proc_macro2::Span;
@@ -29,6 +29,7 @@ const MIGRATIONS_FILENAME: &str = "migrations.toml";
 const MIGRATIONS_FILENAME: &str = "test.migrations.toml";
 
 mod insert;
+mod update;
 
 // trait Ok<T> {
 //  fn ok(self) -> Result<T, anyhow::Error>;
@@ -483,7 +484,7 @@ fn do_parse_tokens(
 
   return Ok(quote! {
   {
-   (|| -> Result<_, _> {
+   (|| -> ::turbosql::Result<usize> {
     ::turbosql::__TURBOSQL_DB.with(|db| {
      let db = db.borrow_mut();
      let mut stmt = db.prepare_cached(#sql)?;
@@ -548,7 +549,7 @@ fn do_parse_tokens(
    quote! {
     {
      // #struct_decl
-     (|| -> Result<Vec<#contents>, ::turbosql::Error> {
+     (|| -> ::turbosql::Result<Vec<#contents>> {
       ::turbosql::__TURBOSQL_DB.with(|db| {
        let db = db.borrow_mut();
        let mut stmt = db.prepare_cached(#sql)?;
@@ -580,12 +581,12 @@ fn do_parse_tokens(
    quote! {
     {
      // #struct_decl
-     (|| -> Result<Option<#contents>, ::turbosql::Error> {
+     (|| -> ::turbosql::Result<Option<#contents>> {
       use ::turbosql::OptionalExtension;
       ::turbosql::__TURBOSQL_DB.with(|db| {
        let db = db.borrow_mut();
        let mut stmt = db.prepare_cached(#sql)?;
-       let result = stmt.query_row(::turbosql::params![#params], |row| -> Result<#contents, _> {
+       let result = stmt.query_row(::turbosql::params![#params], |row| -> ::turbosql::Result<#contents> {
         Ok(#contents {
          #(#row_casters),*
          // #default
@@ -606,11 +607,11 @@ fn do_parse_tokens(
   {
    quote! {
     {
-     (|| -> Result<#contents, ::turbosql::Error> {
+     (|| -> ::turbosql::Result<#contents> {
       ::turbosql::__TURBOSQL_DB.with(|db| {
        let db = db.borrow_mut();
        let mut stmt = db.prepare_cached(#sql)?;
-       let result = stmt.query_row(::turbosql::params![#params], |row| -> Result<#contents, _> {
+       let result = stmt.query_row(::turbosql::params![#params], |row| -> ::turbosql::Result<#contents> {
         Ok(row.get(0)?)
        })?;
        Ok(result)
@@ -629,11 +630,11 @@ fn do_parse_tokens(
 
    quote! {
     {
-     (|| -> Result<#contents, ::turbosql::Error> {
+     (|| -> ::turbosql::Result<#contents> {
       ::turbosql::__TURBOSQL_DB.with(|db| {
        let db = db.borrow_mut();
        let mut stmt = db.prepare_cached(#sql)?;
-       let result = stmt.query_row(::turbosql::params![#params], |row| -> Result<#contents, _> {
+       let result = stmt.query_row(::turbosql::params![#params], |row| -> ::turbosql::Result<#contents> {
         Ok(#contents {
          #(#row_casters),*
          // #default
@@ -682,7 +683,7 @@ pub fn select(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
  proc_macro::TokenStream::from(tokens)
 }
 
-/// Derive this on a `struct` to create a corresponding SQLite table and `insert`/`update`/`upsert` methods. (TODO: `Turbosql` trait?)
+/// Derive this on a `struct` to create a corresponding SQLite table and `Turbosql` trait methods.
 #[proc_macro_derive(Turbosql, attributes(turbosql))]
 #[proc_macro_error]
 pub fn turbosql_derive_macro(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
@@ -732,12 +733,14 @@ pub fn turbosql_derive_macro(input: proc_macro::TokenStream) -> proc_macro::Toke
  // create trait functions
 
  let fn_insert = insert::insert(&table);
+ let fn_update = update::update(&table);
 
  // output tokenstream
 
  proc_macro::TokenStream::from(quote! {
   impl #table {
    #fn_insert
+   #fn_update
   }
  })
 }
@@ -793,8 +796,8 @@ fn extract_columns(fields: &FieldsNamed) -> Vec<Column> {
     (_, "Option < u32 >") => "INTEGER",
     (_, "Option < i54 >") => "INTEGER",
     (_, "Option < i64 >") => "INTEGER",
-    (_, "u64") => abort!(ty, SQLITE_64BIT_ERROR),
-    (_, "Option < u64 >") => abort!(ty, SQLITE_64BIT_ERROR),
+    (_, "u64") => abort!(ty, SQLITE_U64_ERROR),
+    (_, "Option < u64 >") => abort!(ty, SQLITE_U64_ERROR),
     // (_, "f64") => "REAL NOT NULL",
     (_, "Option < f64 >") => "REAL",
     (_, "Option < f32 >") => "REAL",
